***************************************
* Mantel/Haenszel (1959) test statistic
* and Rosenbaum (2002) bounds
***************************************

# delimit ;

capture log close;
clear;

cd "$rootcafe";
log using "logs/MH1959-R2002-macro-hetero-$par.log", replace;
adopath + "$root/%stata/ado"; 

set matsize 800;
set more off;
set rmsg on;
set seed 1234;

capture mkdir "$rootcafe/estmat/$par";
capture mkdir "$rootcafe/Xdata/$par";

# delimit cr

foreach het of global hetlist { /* het */
foreach sample of global samplelist { /* begin of foreach sample */
foreach region in WW APC CEE EMU ODV OIN OWE RCA { /* begin of foreach region */
if `sample'==3 {
global specrange "1 2 3 4"
}
else {
global specrange "1 2 3 4"
}
foreach spec of global specrange { /* begin of foreach spec */


local f 0 

while `f'<=20{ /* begin of grid 1, 1.05, 1.10, 1.15, 1.20, ..., 2 */

   capture noi insheet using "$root/!cafe/Xdata/$par/data_sample`sample'_spec`spec'_het`het'_`region'.csv", comma clear

   if _rc==0 { /* begin of capture-if loop */
   
   keep if (_treated==1|_weight==1)
   keep if _support==1

   qui des
   di r(N)
   if r(N)==0 {
      di "No treated in support, cannot compute Mantel-Haenszel statistic for sample `sample' spec`spec' `region'"
      drop _all
      set obs 1
      gen $outcome=999
      gen _treated=999
   }

   qui tab $outcome if _treated ==1 & $outcome==1
   scalar y_1 = r(N)
   qui tab $outcome if _treated ==0 & $outcome==1
   scalar y_0 = r(N)

   qui tab _treated if _treated==1 
   scalar n_1 = r(N)
   qui tab _treated if _treated==0
   scalar n_0 = r(N)

   qui tab $outcome if $outcome==1
   scalar y = r(N)

   qui tab _treated
   scalar n = r(N)


   drop _all
   set obs 1       


   qui gen double e_y = 1+`f'/20
   qui gen double mh = ((scalar(y_1)-((scalar(n_1)*scalar(y))/scalar(n)))^2) / ( scalar(n_1)*scalar(n_0)*scalar(y)*(scalar(n)-scalar(y)) / (scalar(n)^2*(scalar(n)-1)) )

   qui gen double b = -((e_y - 1) * (scalar(n_1) + scalar(y)) + scalar(n)) /* multiplication instead of addition between first and second terms */
   qui gen double a = (e_y - 1)
   qui gen double c = (e_y*scalar(y)*scalar(n_1))
 
   qui gen double E_over1 = (-b + (b^2-4*a*c)^(0.5))/(2*a)
   qui gen double E_over2 = (-b - (b^2-4*a*c)^(0.5))/(2*a)
 
   qui gen double Var_E_over1 = ( 1/E_over1 + 1/(scalar(y)-E_over1) + 1/(scalar(n_1)-E_over1) + 1 / (scalar(n)-scalar(y)-scalar(n_1)+E_over1))^(-1) /* crucial PLUS sign in last ratio */
   qui gen double Var_E_over2 = ( 1/E_over2 + 1/(scalar(y)-E_over2) + 1/(scalar(n_1)-E_over2) + 1 / (scalar(n)-scalar(y)-scalar(n_1)+E_over2))^(-1) /* crucial PLUS sign in last ratio */

   qui gen double mh_over1 = (scalar(y_1) - E_over1)^2 / Var_E_over1 
   qui gen double mh_over2 = (scalar(y_1) - E_over2)^2 / Var_E_over2 

   qui gen double mh_over1_new = ( abs(scalar(y_1) - E_over1)-0.5 ) / (Var_E_over1)^(0.5)
   qui gen double mh_over2_new = ( abs(scalar(y_1) - E_over2)-0.5 ) / (Var_E_over2)^(0.5)

   qui gen double e_y2 = 1/e_y

   qui gen double b2 = -((e_y2 - 1) * (scalar(n_1) + scalar(y)) + scalar(n)) /* multiplication instead of addition between first and second terms */
   qui gen double a2 = (e_y2 - 1)
   qui gen double c2 = (e_y2*scalar(y)*scalar(n_1))

   qui gen double E_under1 = (-b2 + (b2^2-4*a2*c2)^(0.5))/(2*a2)
   qui gen double E_under2 = (-b2 - (b2^2-4*a2*c2)^(0.5))/(2*a2)

   qui gen double Var_E_under1 = ( 1/E_under1 + 1/(scalar(y)-E_under1) + 1/(scalar(n_1)-E_under1) + 1/(scalar(n)-scalar(y)-scalar(n_1)+E_under1) )^(-1) /* crucial PLUS sign in last ratio */
   qui gen double Var_E_under2 = ( 1/E_under2 + 1/(scalar(y)-E_under2) + 1/(scalar(n_1)-E_under2) + 1/(scalar(n)-scalar(y)-scalar(n_1)+E_under2) )^(-1) /* crucial PLUS sign in last ratio */

   qui gen double mh_under1 = (scalar(y_1) - E_under1)^2 / Var_E_under1 
   qui gen double mh_under2 = (scalar(y_1) - E_under2)^2 / Var_E_under2 

   qui gen double mh_under1_new = ( abs(scalar(y_1) - E_under1)-0.5 ) / (Var_E_under1)^(0.5)
   qui gen double mh_under2_new = ( abs(scalar(y_1) - E_under2)-0.5 ) / (Var_E_under2)^(0.5)
  
   qui gen double p_mh = chi2tail(1,mh)
   qui gen double p_mh_under1 = chi2tail(1,mh_under1)
   qui gen double p_mh_under2 = chi2tail(1,mh_under2)
   qui gen double p_mh_over1 = chi2tail(1,mh_over1)
   qui gen double p_mh_over2 = chi2tail(1,mh_over2)
 
   qui gen double bound1 = max(0,scalar(y)+scalar(n_1)-scalar(n)) 
   qui gen double bound2 = min(scalar(y),scalar(n_1))

   keep e_y mh mh_over1 mh_over2 mh_under1 mh_under2 mh_over1_new mh_over2_new mh_under1_new mh_under2_new p_mh p_mh_under1 p_mh_under2 p_mh_over1 p_mh_over2 E_over1 E_over2 E_under1 E_under2 bound1 bound2

   save "$root/!cafe/Xdata/$par/sensit_sample`sample'_spec`spec'_het`het'_`region'-${outcome}_`f'.dta", replace

   } /* end of capture-if loop */

   else { /* begin of capture-else loop */ 
      clear
      set obs 1
      global statlist "e_y mh mh_over1 mh_over2 mh_under1 mh_under2 mh_over1_new mh_over2_new mh_under1_new mh_under2_new p_mh p_mh_under1 p_mh_under2 p_mh_over1 p_mh_over2 E_over1 E_over2 E_under1 E_under2 bound1 bound2"
      foreach stat of global statlist {
         qui gen `stat' = .
      }
      save "$root/!cafe/Xdata/$par/sensit_sample`sample'_spec`spec'_het`het'_`region'-${outcome}_`f'", replace
   } /* end of capture-else loop */

   local f = `f'+1

} /* end of grid 1, 1.05, 1.10, 1.15, 1.20, ..., 2 */


} /* end of foreach sample */
} /* end of foreach region */
} /* end of foreach spec */
} /* end of het */


/************************************************************************************************/


foreach het of global hetlist { /* het */
foreach sample of global samplelist { /* begin of foreach sample */
foreach region in WW APC CEE EMU ODV OIN OWE RCA { /* begin of foreach region */
if `sample'==3 {
global specrange "1 2 3 4"
}
else {
global specrange "1 2 3 4"
}
foreach spec of global specrange { /* begin of foreach spec */


use "$root/!cafe/Xdata/$par/sensit_sample`sample'_spec`spec'_het`het'_`region'-${outcome}_0.dta", clear

local f 1 

while `f'<=20{ /* begin of grid 1, 1.05, 1.10, 1.15, 1.20, ..., 2 */

   append using "$root/!cafe/Xdata/$par/sensit_sample`sample'_spec`spec'_het`het'_`region'-${outcome}_`f'.dta"
   local f = `f'+1
   
} /* end of grid 1, 1.05, 1.10, 1.15, 1.20, ..., 2 */
   
outsheet using "$root/!cafe/estmat/$par/sensit_sample`sample'_spec`spec'_het`het'_`region'-$outcome.csv", noquote comma replace

} /* end of foreach sample */
} /* end of foreach region */
} /* end of foreach spec */
} /* end of het */

# delimit ;

log close;
